Package: scHOT
Type: Package
Title: single-cell higher order testing
Version: 1.25.0
Authors@R: c(
    person("Shila", "Ghazanfar", email = "shazanfar@gmail.com", role = c("aut", "cre")
    ),
    person("Yingxin", "Lin", email = "yingxin.lin@sydney.edu.au", role = c("aut")
    ))
Description: Single cell Higher Order Testing (scHOT) is an R package that facilitates testing changes in higher order structure of gene expression along either a developmental trajectory or across space. scHOT is general and modular in nature, can be run in multiple data contexts such as along a continuous trajectory, between discrete groups, and over spatial orientations; as well as accommodate any higher order measurement such as variability or correlation. scHOT meaningfully adds to first order effect testing, such as differential expression, and provides a framework for interrogating higher order interactions from single cell data.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.0)
Imports: S4Vectors (>= 0.24.3), SingleCellExperiment, Matrix,
        SummarizedExperiment, IRanges, methods, stats, BiocParallel,
        reshape, ggplot2, igraph, grDevices, ggforce, graphics
RoxygenNote: 7.1.0
Suggests: knitr, markdown, rmarkdown, scater, scattermore, scales,
        matrixStats, deldir
VignetteBuilder: knitr
biocViews: GeneExpression, RNASeq, Sequencing, SingleCell, Software,
        Transcriptomics
git_url: https://git.bioconductor.org/packages/scHOT
git_branch: devel
git_last_commit: a5bc634
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:58:19 UTC; biocbuild
Author: Shila Ghazanfar [aut, cre],
  Yingxin Lin [aut]
Maintainer: Shila Ghazanfar <shazanfar@gmail.com>
