# Generated by roxygen2: do not edit by hand

export("params<-")
export("positionColData<-")
export("positionType<-")
export("scHOT_output<-")
export("testingScaffold<-")
export("weightMatrix<-")
export(plotColouredExpression)
export(plotEgoNetwork)
export(plotHigherOrderSequence)
export(plotOrderedExpression)
export(scHOT)
export(scHOT_addTestingScaffold)
export(scHOT_buildFromMatrix)
export(scHOT_buildFromSCE)
export(scHOT_calculateGlobalHigherOrderFunction)
export(scHOT_calculateHigherOrderTestStatistics)
export(scHOT_estimatePvalues)
export(scHOT_performPermutationTest)
export(scHOT_plotPermutationDistributions)
export(scHOT_setPermutationScaffold)
export(scHOT_setWeightMatrix)
export(scHOT_stripOutput)
export(spatialWeightMatrix)
export(thin)
export(trajectoryWeightMatrix)
export(weightedPearson)
export(weightedSpearman)
export(weightedVariance)
export(weightedZIKendall)
export(weightedZISpearman)
exportClasses(scHOT)
import(ggplot2)
import(igraph)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(S4Vectors,DFrame)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(IRanges,NumericList)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,coolcat)
importFrom(S4Vectors,setValidity2)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(ggforce,geom_voronoi_tile)
importFrom(ggplot2,ggplot)
importFrom(graphics,plot)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,validObject)
importFrom(reshape,melt)
importFrom(reshape,sort_df)
importFrom(stats,dist)
importFrom(stats,loess)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,quantile)
importFrom(stats,sd)
