#' @title Liver trajectory example data
#' @description A liver data set contains two branches
#' (hepatocyte and cholangiocyte).
#' @usage data(liver, package = 'scHOT')
#'
#' @format A `list` object.
"liver"

#' @title MOB_subset spatial example data
#' @description  A MOB spatial data set.
#' @usage data(MOB_subset, package = 'scHOT')
"MOB_subset"
