% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{trajectoryWeightMatrix}
\alias{trajectoryWeightMatrix}
\title{trajectoryWeightMatrix}
\usage{
trajectoryWeightMatrix(n, type = NULL, span = NULL)
}
\arguments{
\item{n}{indicates the number of cels}

\item{type}{Type of weight matrix, one of "triangular" (default),
"block", and "harmonic"}

\item{span}{proportion of samples to include on either side, default is 0.25}
}
\value{
A weighted matrix
}
\description{
Create weight matrix for trajectory data
}
\examples{

W <- trajectoryWeightMatrix(100)
W <- trajectoryWeightMatrix(100, type = "triangular")
W <- trajectoryWeightMatrix(100, type = "block")
W <- trajectoryWeightMatrix(100, type = "harmonic")

}
